/*
 *	wiiuse
 *
 *	Written By:
 *		Michael Laforest	< para >
 *		Email: < thepara (--AT--) g m a i l [--DOT--] com >
 *
 *	Copyright 2006-2007
 *
 *	This file is part of wiiuse.
 *
 *	This program is free software; you can redistribute it and/or modify
 *	it under the terms of the GNU General Public License as published by
 *	the Free Software Foundation; either version 3 of the License, or
 *	(at your option) any later version.
 *
 *	This program is distributed in the hope that it will be useful,
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *	GNU General Public License for more details.
 *
 *	You should have received a copy of the GNU General Public License
 *	along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 *	$Header: /cvsroot/devkitpro/libogc/wiiuse/wiiboard.c,v 1.6 2008/05/26 19:24:53 shagkur Exp $
 *
 */

/**
 *	@file
 *	@brief Wiiboard expansion device.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>

#ifdef WIN32
	#include <Winsock2.h>
#endif

#include "definitions.h"
#include "wiiuse_internal.h"
#include "dynamics.h"
#include "events.h"
#include "wiiboard.h"
#include "io.h"

/**
 *	@brief Handle the handshake data from the wiiboard.
 *
 *	@param cc		A pointer to a classic_ctrl_t structure.
 *	@param data		The data read in from the device.
 *	@param len		The length of the data block, in bytes.
 *
 *	@return	Returns 1 if handshake was successful, 0 if not.
 */
int wii_board_handshake(struct wiimote_t* wm, struct wii_board_t* wb, ubyte* data, uword len) {
	int i;
	int offset = 0;

	if (data[offset] == 0xFF) {
		/*
		 *	Sometimes the data returned here is not correct.
		 *	This might happen because the wiimote is lagging
		 *	behind our initialization sequence.
		 *	To fix this just request the handshake again.
		 *
		 *	Other times it's just the first 16 bytes are 0xFF,
		 *	but since the next 16 bytes are the same, just use
		 *	those.
		 */
		if (data[offset + 16] == 0xFF) {
			/* get the calibration data */
			//printf("Wii board handshake appears invalid, trying again.\n");
			wiiuse_read_data(wm, data, WM_EXP_MEM_CALIBR, EXP_HANDSHAKE_LEN, wiiuse_handshake_expansion);
			return 0;
		} else
			offset += 16;
	}

	short *handshake_short = (short*)data;
	
	wb->ctr[0] = BIG_ENDIAN_SHORT(handshake_short[2]);
	wb->cbr[0] = BIG_ENDIAN_SHORT(handshake_short[3]);
	wb->ctl[0] = BIG_ENDIAN_SHORT(handshake_short[4]);
	wb->cbl[0] = BIG_ENDIAN_SHORT(handshake_short[5]);

	wb->ctr[1] = BIG_ENDIAN_SHORT(handshake_short[6]);
	wb->cbr[1] = BIG_ENDIAN_SHORT(handshake_short[7]);
	wb->ctl[1] = BIG_ENDIAN_SHORT(handshake_short[8]);
	wb->cbl[1] = BIG_ENDIAN_SHORT(handshake_short[9]);

	wb->ctr[2] = BIG_ENDIAN_SHORT(handshake_short[10]);
	wb->cbr[2] = BIG_ENDIAN_SHORT(handshake_short[11]);
	wb->ctl[2] = BIG_ENDIAN_SHORT(handshake_short[12]);
	wb->cbl[2] = BIG_ENDIAN_SHORT(handshake_short[13]);

	/* handshake done */
	wm->exp.type = EXP_WII_BOARD;

	#ifdef WIN32
	wm->timeout = WIIMOTE_DEFAULT_TIMEOUT;
	#endif

	return 1; 
}


/**
 *	@brief The wii board disconnected.
 *
 *	@param cc		A pointer to a classic_ctrl_t structure.
 */
void wii_board_disconnected(struct wii_board_t* wb) {
	memset(wb, 0, sizeof(struct wii_board_t));
}


/**
 *	@brief Handle wii board event.
 *
 *	@param cc		A pointer to a classic_ctrl_t structure.
 *	@param msg		The message specified in the event packet.
 */
void wii_board_event(struct wii_board_t* wb, ubyte* msg) { 
	int i;

	short *shmsg = (short*)(msg);
	wb->rtr = BIG_ENDIAN_SHORT(shmsg[0]);
	//if(wb->rtr<0) wb->rtr = 0;
	wb->rbr = BIG_ENDIAN_SHORT(shmsg[1]);
	//if(wb->rbr<0) wb->rbr = 0;
	wb->rtl = BIG_ENDIAN_SHORT(shmsg[2]);
	//if(wb->rtl<0) wb->rtl = 0;
	wb->rbl = BIG_ENDIAN_SHORT(shmsg[3]);		
	//if(wb->rbl<0) wb->rbl = 0;

	/* 
		Interpolate values 
		Calculations borrowed from wiili.org - No names to mention sadly :( http://www.wiili.org/index.php/Wii_Balance_Board_PC_Drivers page however!
	*/

	if(wb->rtr<wb->ctr[1])
	{
		wb->tr = 68*(wb->rtr-wb->ctr[0])/(wb->ctr[1]-wb->ctr[0]);
	}
	else if(wb->rtr >= wb->ctr[1])
	{
		wb->tr = 68*(wb->rtr-wb->ctr[1])/(wb->ctr[2]-wb->ctr[1]) + 68;
	}

	if(wb->rtl<wb->ctl[1])
	{
		wb->tl = 68*(wb->rtl-wb->ctl[0])/(wb->ctl[1]-wb->ctl[0]);
	}
	else if(wb->rtl >= wb->ctl[1])
	{
		wb->tl = 68*(wb->rtl-wb->ctl[1])/(wb->ctl[2]-wb->ctl[1]) + 68;
	}

	if(wb->rbr<wb->cbr[1])
	{
		wb->br = 68*(wb->rbr-wb->cbr[0])/(wb->cbr[1]-wb->cbr[0]);
	}
	else if(wb->rbr >= wb->cbr[1])
	{
		wb->br = 68*(wb->rbr-wb->cbr[1])/(wb->cbr[2]-wb->cbr[1]) + 68;
	}

	if(wb->rbl<wb->cbl[1])
	{
		wb->bl = 68*(wb->rbl-wb->cbl[0])/(wb->cbl[1]-wb->cbl[0]);
	}
	else if(wb->rbl >= wb->cbl[1])
	{
		wb->bl = 68*(wb->rbl-wb->cbl[1])/(wb->cbl[2]-wb->cbl[1]) + 68;
	}	 
	
	wb->x = (wb->tr+wb->br)/2 - (wb->tl+wb->bl)/2;
	wb->y = (wb->bl+wb->br)/2 - (wb->tl+wb->tr)/2;
	
}


void wiiuse_set_wii_board_calib(struct wiimote_t *wm)
{
} 
